﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;
using WpfValidationDemo.Commands;
using WpfValidationDemo.DataProvider;
using WpfValidationDemo.Model;
using WpfValidationDemo.Wrapper;

namespace WpfValidationDemo.ViewModel
{
  public class FriendEditViewModel : Observable
  {
    private readonly IFriendDataProvider _dataProvider;
    private FriendWrapper _friend;

    public FriendEditViewModel(IFriendDataProvider dataProvider)
    {
      _dataProvider = dataProvider;
      SaveCommand = new DelegateCommand(OnSaveExecute, OnSaveCanExecute);
    }

    public void Load(int friendIdToLoad)
    {
      var friend = _dataProvider.GetFriendById(friendIdToLoad);
      Friend = new FriendWrapper(friend);
      Friend.PropertyChanged += (s, e) =>
        {
          if(e.PropertyName == nameof(Friend.HasErrors))
          {
            ((DelegateCommand)SaveCommand).RaiseCanExecuteChanged();
          }
        };
      ((DelegateCommand)SaveCommand).RaiseCanExecuteChanged();
    }

    public ICommand SaveCommand { get; set; }

    public FriendWrapper Friend
    {
      get { return _friend; }
      set
      {
        _friend = value;
        OnPropertyChanged();
      }
    }

    private void OnSaveExecute(object obj)
    {
      //Friend.Validate();
      //if(Friend.HasErrors)
      //{
      //  MessageBox.Show("Friend hat Fehler");
      //}
      // NOTE: This is just for demo-purpose
      Debug.WriteLine($"Saved: {DateTime.Now:dd.MM.yyyy HH:mm:ss}");
    }

    private bool OnSaveCanExecute(object arg)
    {
      return Friend!=null && !Friend.HasErrors;
    }
  }
}
