﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WpfValidationDemo.Model;
using WpfValidationDemo.ViewModel;

namespace WpfValidationDemo.Wrapper
{
  public class FriendWrapper : NotifyDataErrorBase,IValidatableObject
  {
    private Friend _friend;

    public FriendWrapper(Friend friend)
    {
      _friend = friend;
    }

   
    public string FirstName
    {
      get { return _friend.FirstName; }
      set
      {
        _friend.FirstName = value;
        OnPropertyChanged();

        Validate();
      }
    }

   
    public string LastName
    {
      get { return _friend.LastName; }
      set
      {
        _friend.LastName = value;
        OnPropertyChanged();
        Validate();
      }
    }

    public string Email
    {
      get { return _friend.Email; }
      set
      {
        if (value == string.Empty)
        {
          _friend.Email = null;
        }
        else
        {
          _friend.Email = value;
        }
        OnPropertyChanged();
       Validate();
      }
    }

    public bool IsDeveloper
    {
      get { return _friend.IsDeveloper; }
      set
      {
        _friend.IsDeveloper = value;
        OnPropertyChanged();
        Validate();
      }
    }

    public IEnumerable<ValidationResult> Validate(ValidationContext validationContext)
    {
      if(IsDeveloper && string.IsNullOrEmpty(Email))
      {
        yield return new ValidationResult("Email required for devs!",
          new[] { nameof(IsDeveloper), nameof(Email) });
      }
      if(string.IsNullOrEmpty(FirstName))
      {
        yield return new ValidationResult("Firstname is mandatory!",
         new[] { nameof(FirstName) });
      }
      if(!new EmailAddressAttribute().IsValid(Email))
      {
        yield return new ValidationResult("Email is invalid!",
         new[] { nameof(Email) });
      }
    }
  }
}
