﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using WpfValidationDemo.ViewModel;

namespace WpfValidationDemo.Wrapper
{
  public class NotifyDataErrorBase : Observable, INotifyDataErrorInfo
  {
    private Dictionary<string, List<string>> _errorsByProp
      = new Dictionary<string, List<string>>();

    public bool HasErrors
    {
      get
      {
        return _errorsByProp.Any();
      }
    }

    public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

    public IEnumerable GetErrors(string propertyName)
    {
      return _errorsByProp.ContainsKey(propertyName)
         ? _errorsByProp[propertyName]
         : null;
    }

    protected virtual void OnErrorsChanged(string propertyName)
    {
      ErrorsChanged?.Invoke(this, new DataErrorsChangedEventArgs(propertyName));
    }

    protected virtual void AddError(string propertyName, string error)
    {
      if (!_errorsByProp.ContainsKey(propertyName))
      {
        _errorsByProp[propertyName] = new List<string>();
      }
      if (!_errorsByProp[propertyName].Contains(error))
      {
        _errorsByProp[propertyName].Add(error);
        OnErrorsChanged(propertyName);
        OnPropertyChanged(nameof(HasErrors));
      }
    }

    protected virtual void ClearErrors(string propertyName)
    {
      if (_errorsByProp.ContainsKey(propertyName))
      {
        _errorsByProp.Remove(propertyName);
        OnErrorsChanged(propertyName);
        OnPropertyChanged(nameof(HasErrors));
      }
    }

    protected virtual void ClearAllErrors()
    {
      var propertyNames = _errorsByProp.Keys.ToList();
      foreach (var propertyName in propertyNames)
      {
        ClearErrors(propertyName);
      }
    }

    public void Validate()
    {
      var results = new List<ValidationResult>();
      var context = new ValidationContext(this);
      Validator.TryValidateObject(this, context, results, true);

      ClearAllErrors();
      foreach (ValidationResult result in results)
      {
        foreach (var memberName in result.MemberNames)
        {
          AddError(memberName, result.ErrorMessage);
        }
      }
    }
  }
}
